from typing import Callable, Generator, Tuple

from olympict.types import Img, Size


class ObjPipeline:
    def resize(self, size: Size, count: int = 1):
        """
        Args:
            size (Size): width, height
        """

    def from_generator(self, generator: Generator[Img, None, None]) -> "ObjPipeline":
        raise NotImplementedError()

    def rescale(self, size: Tuple[float, float], count: int = 1) -> "ObjPipeline":
        """
        Args:
            size (Tuple[float, float]): width multiplier, height multiplier
        """
        raise NotImplementedError()

    def crop(self, size: Size, count: int = 1) -> "ObjPipeline":
        """
        Args:
            size (Size): width, height
        """
        raise NotImplementedError()

    def img_task(self, img_func: Callable[[Img], Img]) -> "ObjPipeline":
        raise NotImplementedError()

    def path_task(self, path_func: Callable[[str], str]) -> "ObjPipeline":
        raise NotImplementedError()



from typing import Optional

from olympict.types import Size


class OlympFile:
    def __init__(self, path: Optional[str] = None):
        self.path = path or ""

    @property
    def size(self) -> Size:
        raise NotImplementedError("No size for this object")



import os
from glob import glob
from random import randint
from typing import Any, Callable, Generator, Iterable, List, Optional, Tuple, cast

import cv2  # type: ignore
from olympipe import Pipeline

from olympict.files.o_image import OlympImage
from olympict.files.o_video import OlympVid
from olympict.image_tools import ImTools
from olympict.pipeline import OPipeline
from olympict.types import (
    BBoxAbsolute,
    BBoxRelative,
    Color,
    Img,
    ImgFormat,
    LineAbsolute,
    LineRelative,
    PolygonAbsolute,
    PolygonRelative,
    Size,
)
from olympict.video_saver import PipelineVideoSaver


class ImagePipeline(OPipeline[OlympImage]):
    def __init__(self, data: Optional[Iterable[OlympImage]] = None):
        self.pipeline: Optional[Pipeline[OlympImage]] = None

        if data is not None:
            self.pipeline = Pipeline(data)

    @staticmethod
    def load_paths(
        paths: List[str],
        metadata_function: Optional[Callable[[str], Any]] = None,
    ) -> "ImagePipeline":
        data = [OlympImage(p) for p in paths]

        if metadata_function is not None:
            for d in data:
                d.metadata = metadata_function(d.path)

        return ImagePipeline(data)

    @staticmethod
    def load_folder(
        path: str,
        extensions: List[str] = ["png", "jpg", "jpeg", "bmp"],
        recursive: bool = False,
        order_func: Optional[Callable[[str], int]] = None,
        reverse: bool = False,
        metadata_function: Optional[Callable[[str], Any]] = None,
    ) -> "ImagePipeline":
        paths: List[str] = glob(os.path.join(path, "**"), recursive=recursive)

        paths = [p for p in paths if os.path.splitext(p)[1].strip(".") in extensions]

        if order_func is not None:
            paths.sort(key=order_func, reverse=reverse)

        data = [OlympImage(p) for p in paths]

        if metadata_function is not None:
            for d in data:
                d.metadata = metadata_function(d.path)

        return ImagePipeline(data)

    @staticmethod
    def load_folders(
        paths: List[str],
        extensions: List[str] = ["png", "jpg", "jpeg", "bmp"],
        recursive: bool = False,
        order_func: Optional[Callable[[str], int]] = None,
        reverse: bool = False,
        metadata_function: Optional[Callable[[str], Any]] = None,
    ) -> "ImagePipeline":
        all_data: List[OlympImage] = []
        for path in paths:
            sub_paths: List[str] = glob(os.path.join(path, "**"), recursive=recursive)

            sub_paths = [
                p for p in sub_paths if os.path.splitext(p)[1].strip(".") in extensions
            ]

            if order_func is not None:
                sub_paths.sort(key=order_func, reverse=reverse)

            data = [OlympImage(p) for p in sub_paths]

            if metadata_function is not None:
                for d in data:
                    d.metadata = metadata_function(d.path)

            all_data.extend(data)

        return ImagePipeline(all_data)

    def task(
        self, func: Callable[[OlympImage], OlympImage], count: int = 1
    ) -> "ImagePipeline":
        output = ImagePipeline()
        if self.pipeline is None:
            raise Exception("Undefined pipeline")
        output.pipeline = self.pipeline.task(func, count)

        return output

    def task_img(self, func: Callable[[Img], Img], count: int = 1) -> "ImagePipeline":
        def r(o: OlympImage) -> OlympImage:
            o.img = func(o.img)
            return o

        return self.task(r, count)

    def task_path(self, func: Callable[[str], str], count: int = 1) -> "ImagePipeline":
        def r(o: OlympImage) -> OlympImage:
            o.ensure_load()
            o.path = func(o.path)
            return o

        return self.task(r, count)

    def explode(self, explode_function: Callable[[OlympImage], List[OlympImage]]):
        if self.pipeline is None:
            raise Exception("No defined pipeline")

        output = ImagePipeline()
        output.pipeline = self.pipeline.explode(explode_function)

        return output

    def rescale(
        self,
        size: Tuple[float, float],
        pad_color: Optional[Tuple[int, int, int]] = None,
        count: int = 1,
    ) -> "ImagePipeline":
        return self.task(OlympImage.rescale(size, pad_color), count)

    def resize(
        self,
        size: Tuple[int, int],
        pad_color: Optional[Tuple[int, int, int]] = None,
        interpolation: int = cv2.INTER_LINEAR,
        count: int = 1,
    ) -> "ImagePipeline":
        return self.task(OlympImage.resize(size, pad_color, interpolation), count)

    def crop(
        self,
        left: int = 0,
        top: int = 0,
        right: int = 0,
        bottom: int = 0,
        pad_color: Color = (0, 0, 0),
    ) -> "ImagePipeline":
        def r(img: Img) -> Img:
            return ImTools.crop_image(
                img, top=top, left=left, bottom=bottom, right=right, pad_color=pad_color
            )

        return self.task_img(r)

    def random_crop(
        self,
        output_size: Size,
    ) -> "ImagePipeline":
        def r(img: Img) -> Img:
            h, w, _ = img.shape
            t_w, t_h = output_size

            off_x = randint(0, w - t_w - 1)
            off_y = randint(0, h - t_h - 1)

            return img[off_y : off_y + t_h, off_x : off_x + t_w, :]

        return self.task_img(r)

    def filter(self, filter_function: Callable[[OlympImage], bool]) -> "ImagePipeline":
        if self.pipeline is None:
            raise Exception("No defined pipeline")

        output = ImagePipeline()
        output.pipeline = self.pipeline.filter(filter_function)

        return output

    def keep_each_frame_in(
        self, keep_n: int = 1, discard_n: int = 0
    ) -> "ImagePipeline":
        if self.pipeline is None:
            raise Exception("No defined pipeline")

        def discarder():
            while True:
                for _ in range(keep_n):
                    yield True
                for _ in range(discard_n):
                    yield False

        d = discarder()

        def get_next(_: Any) -> bool:
            return next(d)

        output = ImagePipeline()
        output.pipeline = self.pipeline.filter(get_next)

        return output

    def debug_window(self, name: str) -> "ImagePipeline":
        def d(o: "OlympImage") -> "OlympImage":
            cv2.imshow(name, o.img)
            cv2.waitKey(1)
            return o

        return self.task(d)

    def to_video(
        self, img_to_video_path: Callable[[OlympImage], str], fps: int = 25
    ) -> "VideoPipeline":
        output: VideoPipeline = VideoPipeline()
        output.pipeline = self.pipeline.class_task(  # type: ignore
            PipelineVideoSaver,
            PipelineVideoSaver.process_file,
            [img_to_video_path, fps],
            PipelineVideoSaver.finish,
        )

        return output

    def to_format(self, format: ImgFormat) -> "ImagePipeline":
        def change_format(path: str) -> str:
            base, _ = os.path.splitext(path)

            fmt = f".{format}" if "." != format[0] else format

            return base + fmt

        return self.task_path(change_format)

    def save_to_folder(self, folder_path: str) -> "ImagePipeline":
        os.makedirs(folder_path, exist_ok=True)

        def s(o: "OlympImage") -> "OlympImage":
            o.change_folder_path(folder_path)
            o.save()
            return o

        return self.task(s)

    def save(self) -> "ImagePipeline":
        def s(o: "OlympImage") -> "OlympImage":
            o.save()
            return o

        return self.task(s)

    def draw_relative_polygons(
        self,
        polygon_function: Callable[[OlympImage], List[Tuple[PolygonRelative, Color]]],
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            outputs = polygon_function(o)
            for polygon, color in outputs:
                o.img = ImTools.draw_relative_polygon(o.img, polygon, color)
            return o

        return self.task(p)

    def draw_polygons(
        self,
        polygon_function: Callable[[OlympImage], List[Tuple[PolygonAbsolute, Color]]],
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            outputs = polygon_function(o)
            for polygon, color in outputs:
                o.img = ImTools.draw_polygon(o.img, polygon, color)
            return o

        return self.task(p)

    def draw_relative_bboxes(
        self,
        bbox_function: Callable[[OlympImage], List[Tuple[BBoxRelative, Color]]],
        font_scale: float = ImTools.font_scale,
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            outputs = bbox_function(o)
            for polygon, color in outputs:
                o.img = ImTools.draw_relative_bbox(o.img, polygon, color, font_scale)
            return o

        return self.task(p)

    def draw_bboxes(
        self,
        bbox_function: Callable[[OlympImage], List[Tuple[BBoxAbsolute, Color]]],
        font_scale: float = ImTools.font_scale,
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            for polygon, color in bbox_function(o):
                o.img = ImTools.draw_bbox(o.img, polygon, color, font_scale)
            return o

        return self.task(p)

    def draw_relative_lines(
        self,
        polyline_function: Callable[[OlympImage], List[Tuple[LineRelative, Color]]],
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            outputs = polyline_function(o)
            for line, color in outputs:
                o.img = ImTools.draw_relative_line(o.img, line, color)
            return o

        return self.task(p)

    def draw_lines(
        self,
        polyline_function: Callable[[OlympImage], List[Tuple[LineAbsolute, Color]]],
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            outputs = polyline_function(o)
            for line, color in outputs:
                o.img = ImTools.draw_line(o.img, line, color)
            return o

        return self.task(p)

    def draw_heatmap(
        self, heatmap_function: Callable[[OlympImage], Img]
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            outputs = heatmap_function(o)
            o.img = ImTools.draw_heatmap(o.img, outputs)
            return o

        return self.task(p)

    def draw_segmentation_maps(
        self, segmentation_map: Callable[[OlympImage], Img], color: Color
    ) -> "ImagePipeline":
        def p(o: "OlympImage") -> "OlympImage":
            seg_map = segmentation_map(o)
            o.img = ImTools.draw_segmentation_map(o.img, seg_map, color)
            return o

        return self.task(p)


class VideoPipeline(OPipeline[OlympVid]):
    def __init__(self, data: Optional[Iterable[OlympVid]] = None):
        self.pipeline: Optional[Pipeline[OlympVid]] = None
        if data is not None:
            self.pipeline = Pipeline(data)

    @staticmethod
    def load_folder(
        path: str,
        extensions: List[str] = ["mkv", "mp4"],
        recursive: bool = False,
        order_func: Optional[Callable[[str], int]] = None,
        reverse: bool = False,
    ) -> "VideoPipeline":
        paths: List[str] = glob(os.path.join(path, "**"), recursive=recursive)
        paths = [p for p in paths if os.path.splitext(p)[1].strip(".") in extensions]

        if order_func is not None:
            paths.sort(key=order_func, reverse=reverse)

        data = [OlympVid(p) for p in paths]

        return VideoPipeline(data)

    def task(
        self, func: Callable[[OlympVid], OlympVid], count: int = 1
    ) -> "VideoPipeline":
        return self.task(func, count)

    def move_to_folder(self, folder_path: str) -> "VideoPipeline":
        os.makedirs(folder_path, exist_ok=True)

        def s(o: "OlympVid") -> "OlympVid":
            o.change_folder_path(folder_path)
            return o

        return self.task(s)

    def to_sequence(self) -> "ImagePipeline":
        if self.pipeline is None:
            raise Exception("No defined pipeline")

        def generator(o: "OlympVid") -> Generator[OlympImage, None, None]:
            capture: Any = cv2.VideoCapture(o.path)
            res, frame = cast(Tuple[bool, Img], capture.read())
            idx = 0
            while res:
                new_path = f"{o.path}_{idx}.png"
                yield OlympImage.from_buffer(
                    frame, new_path, {"video_path": o.path, "video_frame": idx}
                )
                res, frame = cast(Tuple[bool, Img], capture.read())
                idx += 1

        output = ImagePipeline()
        output.pipeline = self.pipeline.explode(generator)

        return output



import os
import shutil
from typing import Any, Callable, Dict, Optional, Tuple

import cv2  # type: ignore
import numpy as np

from olympict.files.o_file import OlympFile
from olympict.image_tools import ImTools
from olympict.types import Color, Img, Size


class OlympImage(OlympFile):
    __id = 0

    def __init__(self, path: Optional[str] = None):
        super().__init__(path)
        self._img: Optional[Img] = None
        if path is None:
            self.path = f"./{self.__id}.png"
            self.__id += 1
            self._img = np.zeros((1, 1, 3), dtype=np.uint8)
        self.metadata: Dict[str, Any] = {}

    @property
    def img(self) -> Img:
        self.ensure_load()
        return self._img

    @img.setter
    def img(self, image: Img):
        self._img = image

    def ensure_load(self):
        if self._img is None:
            self._img = cv2.imread(self.path)

    @staticmethod
    def resize(
        size: Size,
        pad_color: Optional[Color] = None,
        interpolation: int = cv2.INTER_LINEAR,
    ) -> Callable[["OlympImage"], "OlympImage"]:
        def r(o: "OlympImage") -> "OlympImage":
            if pad_color is None:
                o.img = cv2.resize(o.img, size, interpolation=interpolation)
            else:
                o.img = ImTools.pad_to_output_size(
                    o.img, size, pad_color, interpolation=interpolation
                )
            return o

        return r

    @staticmethod
    def rescale(
        scales: Tuple[float, float],
        pad_color: Optional[Color] = None,
        interpolation: int = cv2.INTER_LINEAR,
    ) -> Callable[["OlympImage"], "OlympImage"]:
        """Rescale function
        This function applies a scale [x_s, y_s] to a given image
        The resulting image dimensions are [w * x_s, h * y_s]
        """

        def r(o: "OlympImage") -> "OlympImage":
            w, h = o.size
            x_scale, y_scale = scales
            size = (int(round(w * x_scale)), int(round(h * y_scale)))
            if pad_color is None:
                o.img = cv2.resize(o.img, size, interpolation=interpolation)
            else:
                o.img = ImTools.pad_to_output_size(
                    o.img, size, pad_color, interpolation=interpolation
                )
            return o

        return r

    def move_to_path(self, path: str):
        """This function moves images to a new location. If path is a directory, then it will keep its old name and move to the new directory.
        Else it will be given path as a new name (This might be bad for multiple images).
        """
        #  TODO: Ensure folder or not
        if os.path.isdir(path):
            _, filename = os.path.split(self.path)
            path = os.path.join(path, filename)
        shutil.move(self.path, path)
        self.path = os.path.abspath(path)

    def change_folder_path(self, new_folder_path: str):
        self.ensure_load()
        self.path = os.path.join(new_folder_path, os.path.basename(self.path))

    def move_to(self, func: Callable[[str], str]):
        self.ensure_load()
        output = func(self.path)
        self.move_to_path(output)

    def save(self):
        self.ensure_load()
        os.makedirs(os.path.dirname(self.path), exist_ok=True)
        _ = cv2.imwrite(self.path, self.img)

    def save_as(self, path: str):
        self.ensure_load()
        if os.path.isdir(path):
            _, filename = os.path.split(self.path)
            path = os.path.join(path, filename)

        self.path = os.path.abspath(path)
        self.save()

    @property
    def size(self) -> Size:
        h, w, _ = self.img.shape
        return (w, h)

    @staticmethod
    def load(path: str, metadata: Dict[str, Any] = {}) -> "OlympImage":
        o = OlympImage()
        o.path = path
        o.img = cv2.imread(o.path)
        o.metadata = metadata
        return o

    @staticmethod
    def from_buffer(
        buffer: Img, path: str = "", metadata: Dict[str, Any] = {}
    ) -> "OlympImage":
        o = OlympImage()
        o.path = path
        o.img = buffer
        o.metadata = metadata
        return o



from typing import List

import cv2  # type: ignore
import numpy as np

from olympict.types import (
    BBoxAbsolute,
    BBoxRelative,
    Color,
    Img,
    LineAbsolute,
    LineRelative,
    PointAbsolute,
    PointRelative,
    PolygonAbsolute,
    PolygonRelative,
    Size,
)


class ImTools:
    font = 0
    font_scale = 0.4
    blue: Color = (255, 0, 0)
    green: Color = (0, 255, 0)
    red: Color = (0, 0, 255)
    black: Color = (0, 0, 0)
    grey: Color = (127, 127, 127)
    white: Color = (255, 255, 255)

    @staticmethod
    def load(path: str) -> Img:
        return cv2.imread(path)

    @staticmethod
    def load_multiple(paths: List[str]) -> List[Img]:
        return [ImTools.load(path) for path in paths]

    @staticmethod
    def pad_square(image: Img, color: Color = (0, 0, 0)) -> Img:
        """
        Pads an image to fit in a square
        Args:
            image (Image): cv2.Image
        Returns:
            Image: cv2.Image
        """
        max_dim = max(image.shape[:-1])
        padded_image = ImTools.pad_to_output_size(image, (max_dim, max_dim), color)
        return padded_image

    @staticmethod
    def pad_to_ratio(image: Img, ratio: float, color: Color = (0, 0, 0)) -> Img:
        """
        Pads an image to the specified ratio

        Args:
            image (Image): cv2.Image
            ratio (float): width / height

        Returns:
            Image: cv2.Image
        """
        height, width = image.shape[:-1]
        image_ratio = width / height
        if image_ratio < ratio:
            # keep height
            dims = (height, int(round(height * ratio)))
        else:
            # keep width
            dims = (int(round(width / ratio)), width)

        return ImTools.pad_to_output_size(image, dims, color)

    @staticmethod
    def pad_to_output_size(
        image: Img,
        size: Size,
        color: Color = (0, 0, 0),
        interpolation: int = cv2.INTER_LINEAR,
    ) -> Img:
        """Pads an image to the specified size (adds black pixels)

        Args:
            image (Image): cv2.Image
            size (Size): height, width

        Returns:
            Image: cv2.Image
        """
        height, width = image.shape[:-1]

        out_h, out_w = size

        desired_ratio = out_w / out_h
        input_ratio = width / height

        if input_ratio > desired_ratio:
            resized = cv2.resize(
                image, (out_w, int(out_w / input_ratio)), interpolation=interpolation
            )
        else:
            resized = cv2.resize(
                image, (int(out_h * input_ratio), out_h), interpolation=interpolation
            )

        resized_h, resized_w, _ = resized.shape

        pad_top = int((out_h - resized_h) / 2)
        pad_left = int((out_w - resized_w) / 2)
        pad_bottom = int(out_h - pad_top - resized_h)
        pad_right = int(out_w - pad_left - resized_w)

        padded_image = cv2.copyMakeBorder(
            resized,
            pad_top,
            pad_bottom,
            pad_left,
            pad_right,
            cv2.BORDER_CONSTANT,
            value=color,
        )

        return padded_image

    @staticmethod
    def crop_image(
        image: Img,
        left: int = 0,
        top: int = 0,
        right: int = 0,
        bottom: int = 0,
        pad_color: Color = (0, 0, 0),
    ) -> Img:
        """Crops an image. It can also pad an image if numbers are < 0

        Args:
            image (Image): Input cv2 image
            top (int, optional): pixels to remvoe from the top. Defaults to 0.
            left (int, optional): pixels to remvoe from the left. Defaults to 0.
            bottom (int, optional): pixels to remvoe from the bottom. Defaults to 0.
            right (int, optional): pixels to remvoe from the right. Defaults to 0.
            pad_color (Color, optional): Color to use if padding.

        Returns:
            Image: The cv2 image cropped
        """

        h, w, channels = image.shape

        new_h = h - top - bottom
        new_w = w - left - right

        img = np.ones((new_h, new_w, channels), dtype=np.uint8) * pad_color

        left_source = max(left, 0)
        left_destination = max(-left, 0)

        top_source = max(top, 0)
        top_destination = max(-top, 0)

        copy_w = w - max(left, 0) - max(right, 0)
        copy_h = h - max(top, 0) - max(bottom, 0)

        img[
            top_destination : top_destination + copy_h,
            left_destination : left_destination + copy_w,
            :,
        ] = image[
            top_source : top_source + copy_h,
            left_source : left_source + copy_w,
            :,
        ]

        return img

    @staticmethod
    def draw_heatmap(image: Img, map: Img, alpha_map: float = 0.5) -> Img:
        """
        Args:
            image (Image): cv2.image (HWC)
            map (Image): [0-255] cv2.image (HW)
            alpha_map (float): [0-1] balance between image and map
        """

        heatmap_img = cv2.applyColorMap(np.array(map, dtype=np.uint8), cv2.COLORMAP_JET)

        heatmap_img = cv2.resize(
            heatmap_img, image.shape[:2][::-1], interpolation=cv2.INTER_LINEAR
        )

        output = cv2.addWeighted(
            heatmap_img,
            alpha_map,
            image,
            1 - alpha_map,
            0,
            dtype=cv2.PARAM_UNSIGNED_INT,
        )

        return output

    @staticmethod
    def draw_segmentation_map(image: Img, map: Img, color: Color) -> Img:
        """
        Args:
            image (Image): cv2.image (HWC)
            map (Image): [0/1] bool cv2.image (HW)
            color (Color)
        """

        output = image.copy()

        output[map] = color

        return output

    @staticmethod
    def draw_multiple_heatmaps(image: Img, maps: Img, alpha_map: float = 0.5) -> Img:
        """
        Args:
            image (Image): cv2.image (HWC)
            map (Image): [0-255] cv2.image (HW)
            alpha_map (float): [0-1] balance between image and map
        """
        max_map: Img = np.max(maps, axis=-1)
        return ImTools.draw_heatmap(image, max_map, alpha_map=alpha_map)

    @staticmethod
    def convert_point_relative_to_absolute(
        object: PointRelative, w: int, h: int
    ) -> PointAbsolute:
        x, y = object
        return (int(round(x * w)), int(round(y * h)))

    @staticmethod
    def convert_point_absolute_to_relative(
        object: PointAbsolute, w: int, h: int
    ) -> PointRelative:
        x, y = object
        return (float(x) / w, float(y) / h)

    @staticmethod
    def convert_bbox_relative_to_absolute(
        object: BBoxRelative, w: int, h: int
    ) -> BBoxAbsolute:
        x1, y1, x2, y2, class_name, confidence = object
        return (
            *ImTools.convert_point_relative_to_absolute((x1, y1), w, h),
            *ImTools.convert_point_relative_to_absolute((x2, y2), w, h),
            class_name,
            confidence,
        )

    @staticmethod
    def convert_bbox_absolute_to_relative(
        object: BBoxAbsolute, w: int, h: int
    ) -> BBoxRelative:
        x1, y1, x2, y2, class_name, confidence = object
        return (
            *ImTools.convert_point_absolute_to_relative((x1, y1), w, h),
            *ImTools.convert_point_absolute_to_relative((x2, y2), w, h),
            class_name,
            confidence,
        )

    @staticmethod
    def draw_line(image: Img, line: LineAbsolute, color: Color, size: float = 1) -> Img:
        for i in range(len(line) - 1):
            from_point = line[i]
            to_point = line[i + 1]
            image = cv2.line(image, from_point, to_point, color, thickness=size)
        return image

    @staticmethod
    def draw_relative_line(
        image: Img, line: LineRelative, color: Color, size: float = 1
    ) -> Img:
        h, w, _ = image.shape

        abs_line = [ImTools.convert_point_relative_to_absolute(p, w, h) for p in line]

        return ImTools.draw_line(image, abs_line, color, size)

    @staticmethod
    def draw_bbox(
        image: Img, bbox: BBoxAbsolute, color: Color, font_scale: float
    ) -> Img:
        x1, y1, x2, y2, class_name, conf = bbox
        image = cv2.rectangle(image, (x1, y1), (x2, y2), color=color, thickness=1)

        image = cv2.putText(
            image,
            f"{class_name}: {conf:0.2f}%",
            (x1 + int(1 + 0.3 * font_scale), y1 + int(20 * font_scale)),
            ImTools.font,
            font_scale,
            color,
            1,
            cv2.LINE_AA,
        )
        return image

    @staticmethod
    def draw_relative_bbox(
        image: Img, bbox: BBoxRelative, color: Color, font_scale: float
    ) -> Img:
        h, w, _ = image.shape

        abs_bbox = ImTools.convert_bbox_relative_to_absolute(bbox, w, h)

        return ImTools.draw_bbox(image, abs_bbox, color, font_scale)

    @staticmethod
    def draw_polygon(image: Img, polygon: PolygonAbsolute, color: Color) -> Img:
        image = cv2.polylines(image, [np.array(polygon)], True, color, thickness=1)

        return image

    @staticmethod
    def draw_relative_polygon(
        image: Img, polygon: PolygonRelative, color: Color
    ) -> Img:
        h, w, _ = image.shape

        abs_polygon = [
            ImTools.convert_point_relative_to_absolute(p, w, h) for p in polygon
        ]

        return ImTools.draw_polygon(image, abs_polygon, color)



from olympict.pipelines import ImagePipeline, VideoPipeline
from olympict.files.o_image import OlympImage
from olympict.files.o_video import OlympVid
from olympict.types import (
    BBoxAbsolute,
    BBoxRelative,
    Color,
    Img,
    LineAbsolute,
    LineRelative,
    PointAbsolute,
    PointRelative,
    PolygonAbsolute,
    PolygonRelative,
    Size,
)



from typing import Generic, Iterable, List, Optional, TypeVar

from olympipe import Pipeline

T = TypeVar("T")


class OPipeline(Generic[T]):
    def __init__(self, source: Iterable[T]):
        self._data = source
        self.pipeline: Optional[Pipeline[T]] = None

    @staticmethod
    def load_folder(
        path: str, extensions: List[str] = [""], recursive: bool = False
    ) -> "OPipeline[T]":
        raise NotImplementedError()

    def wait_for_completion(self) -> None:
        if self.pipeline is None:
            return
        return self.pipeline.wait_for_completion()

    def wait_for_results(self) -> List[T]:
        if self.pipeline is None:
            return []
        return self.pipeline.wait_for_result()
